/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "llog.h"
#include "libsl.h"

#define DEFAULT_PORT 54321

static int timeout = 5 * 60;

/************************************************************************/
/*                                                                      */
/*  sl_Setup_Client_Socket                                              */
/*                                                                      */
/*  Returns an "Socket Descripter" or "SD" that is and is being         */
/*  The SD will be in "listen" mode.  On error return 0.                */
/*                                                                      */
/*  Input Parameters:                                                   */
/*      host -  If not mull then names a hostname.  If null then the    */
/*              default "localhost" will beused.                        */
/*                                                                      */
/*      port -  If a non-zero port is given that port number will       */
/*              be used.  If port==0 then "service" will be used.       */
/*                                                                      */
/*      service - The name of a TCP service.  as listed in the file     */
/*              /etc/services or in the NIS "services" map.             */
/*              If port==0 then we will try to look uop the service     */
/*              name o get the port number.                             */
/*                                                                      */
/*      If both port==0 and service==NULL then a default port number    */
/*      is used.  Thisis likely only good for development testing       */
/*      as the portnumber should not be used in the "real world".       */
/************************************************************************/


int sl_Setup_Client_Socket(char *hostname, int port, char *service)
{
    int    sd = 0;
    char   default_server[] = "localhost";
    struct sockaddr_in pin;
    struct hostent *hp;
    struct servent *my_service;

    int connected_to_server = 0;
    int wait_seconds = 0;
    int sleep_seconds = 5;
    int unslept_seconds;
    
    char  *Server;
    
    
    /********************************************************************/
    /* Get the server host name.                                        */
    /* If ServerHostname is not NULL is it.  If it is NULL use the      */
    /* defualt which is "localhost".                                    */
    /********************************************************************/
    if( !hostname )
    {
        Server = default_server;
    }
    else if ('\0' == hostname[0])
    {
        Server = default_server;
    }
    else
    {
        Server = hostname;
    }
    

    /********************************************************************/
    /* We now have a hostname in text but we need the address of the    */
    /* server.  So go get a hostent structure.                          */
    /* The gethostbyname function will use whatever service is required */
    /* to find a host, /etc/hosts, NIS or DNS. So the host could be     */
    /* on the local machine or anywhere out on the Internet.            */
    /* Either way we get the IP address and stuff it into a socket      */
    /* address structure.                                               */
    /********************************************************************/
    if ((hp = gethostbyname(Server)) == 0)
    {
        /* Hostname lookup failed.  Print error and bail */
        Llog_warning("Hostname lookup failed, %s\n",
            strerror(errno));
        return(-1);
    }

    /* fill in the socket name with host information */
    memset(&pin, 0, sizeof(pin));
    pin.sin_family = AF_INET;
    pin.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;

    
    /********************************************************************/
    /* Get the port number.                                             */
    /*      If specified by caller (port >0) use it.                    */
    /*      If unspecied try to look it up by using the service name    */
    /*      If the lookup fails or if no service is given use a default */
    /*      port number.                                                */
    /********************************************************************/
    if (port > 0)
    {
        /* port was given.  Use it. */
        pin.sin_port = htons( (short int)port );
        Llog_debug("Given port number\n");
    }
    else if (service)
    {
        if ((my_service = getservbyname(service, "tcp")) == NULL)
        {
            /* Failed to find the tass service.  Maybe the */
            /* /etc/services file does not have a tass     */
            /* entry in it?                                */
            Llog_warning("Failed to find service '%s'\n", service);
            return(-1);
        }
        pin.sin_port = my_service->s_port;
        Llog_debug("Got port number from /etc/services file\n");
    }
    else
    {
        Llog_debug("using default port\n");
        pin.sin_port = htons( (short int)DEFAULT_PORT );
    }

    Llog_verbose("Using port %d \n", ntohs(pin.sin_port));
    
    
       /********************************************************************/
    /* Create an Internet Domain Socket and atempt to connect with      */
    /* another one of these owned by the server.  All of the above was  */
    /* to locate that server.  Now we do the create and connect.        */
    /********************************************************************/



    /********************************************************************/
    /* Try to connect to the tassm3 server on tassm3_host.              */
    /* There are many ways this can fail but the most likey is that the */
    /* the server is simply not running.  So for now we will just keep  */
    /* trying until we reach a timeout limit.                           */
    /* Later I'll add code to check for unrecoverable errors and bail   */
    /* out without haveing to wait for a potentialy very long timeout   */
    /********************************************************************/
    while( !connected_to_server && (wait_seconds < timeout) )
    {
        /****************************************************************/
        /* Create an Internet Domain Socket and atempt to connect with  */
        /* another one of these owned by the server.                    */
        /****************************************************************/

        /* The "socket" function will create a socket */
        sd = socket(AF_INET, SOCK_STREAM, 0);
        if (-1 == sd)
            {
                /* If the above call fails we are in deep you know */
                /* what.  Print an error  and bail.                */
                perror("socket");
                return -1;
            }
    
        if ( -1 == connect(sd, (struct sockaddr *)&pin, sizeof(pin)))
        {
            
            /* Failed to connect.  Likely server is not running */
            Llog_debug("Failed to connect with server, %s\n",
                strerror(errno));
            
            if ( 0 )                    /* <<<<<<<<<<<<<< fix later     */
            {
                /* non-recoverable error */
                /* print a message       */
                return(-1);
            }
            else
            {
                /* recoverable error, retry */
                unslept_seconds = sleep(sleep_seconds);
                wait_seconds = wait_seconds + sleep_seconds
                               - unslept_seconds;
            }
            
            close(sd);
        }
        else
        {
            /* Connected!! */
            connected_to_server = 1;
        }
    }
 

    /********************************************************************/
    /* Return to caller with either:                                    */
    /*      A) a valid socket. or B) sd = 0                             */
    /********************************************************************/
    if (! connected_to_server)
    {
        /* Failed to connect.  Return error */
        return -1;
    }
    
    /* we made it!  Connected. */
    return sd;
}


