
	/*
	 * Routines that deal with the Digital-to-Analog-Converters (DACs)
	 * and Analog-to-Digital-Converters (ADCs).  These convert
	 * from analog voltages to digital readings.
	 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "tait.h"


	/*************************************************************
	 * PROCEDURE: load_dacs
	 *
	 * DESCRIPTION:
	 * 
	 *  This routine loads all the Digital-to-Analog-Conversion (DAC) units
	 *    in the STAMP.  I believe that this puts the proper conversions
	 *    into place, so that (digital) signals sent to the hardware
	 *    are transformed into the correct (analog) voltages first.
	 *
	 *  The routine reads values from the global array g_voltages[],
	 *    scales them into appropriate form, and sets them one at
	 *    a time by sending a message to the STAMP.
	 * 
	 *  One must call this routine on startup, or else some of the
	 *    data returned by both the ADCs and the CCD controller will
	 *    not have the proper values.
	 * 
	 * RETURNS:
	 *    0             if all goes well
	 *    1             if an error occurs
	 */

int
load_dacs
	(
	void
	)
{
	int i, number;
	int value = -1;
	int temp3, temp4;
	int this_voltage;
	char msg[MSG_LEN+1], reply[MSG_LEN+1];	
	
	strcpy(msg, "51xxxxx5");
	msg[MSG_LEN] = '\0';

	/* 
	 * now, loop over all DACs.  For each one, we build up a 
	 * complete message for the STAMP, then send it.
	 */
	for (i = 0; i < NUM_DACS; i++) {

		/* set two characters in the message we send to STAMP */
		if (i < 8) {
			msg[2] = '0';
			msg[3] = '0';
		} 
		else {
			msg[2] = '1';
			msg[3] = '0';
		}

        	/* get value to load in dac address positions */
        	number = i % 8;
        	msg[4] = '0' + 2*number;

       		/* convert the voltage to a DAC setting value */
        	this_voltage = g_voltages[i];
        	value = 140 - ((this_voltage - .05) / .0974);
		if (value > 255) {
			value = 255;
		}
		if (value < 0) {
			value = 0;
		}
		
		/* convert the DAC setting value into a 2-character code */
		temp3 = floor(value/16);
		temp4 = floor(value - temp3*16);
		msg[5] = '0' + temp3;
		msg[6] = '0' + temp4;

		send_message(msg, reply, "load_dacs");
	}

	return(0);
}


	/*************************************************************
	 * PROCEDURE: load_one_dac
	 *
	 * DESCRIPTION:
	 * 
	 *  This routine loads a single Digital-to-Analog-Conversion (DAC) 
	 *    units into the STAMP.  I believe that this puts the 
	 *    proper conversions into place, so that (digital) 
	 *    signals sent to the hardware
	 *    are transformed into the correct (analog) voltages first.
	 *
	 *  The routine takes a value in volts (I think),
	 *    scales it into appropriate form, and sets it
	 *    by sending a message to the STAMP.
	 *
	 *  This routine also changes the value of the appropriate 
	 *    DAC in the global g_voltages[] array.
	 * 
	 * RETURNS:
	 *    0             if all goes well
	 *    1             if an error occurs
	 */

int
load_one_dac
	(
	int dac_index,           /* I: which DAC to set? */
	double voltage           /* I: value to set (volts) */
	)
{
	int number;
	int value = -1;
	int temp3, temp4;
	int this_voltage;
	char msg[MSG_LEN+1], reply[MSG_LEN+1];	

	/* sanity check */
	if ((dac_index < 0) || (dac_index >= NUM_DACS)) {
		shError("load_one_dac: invalid DAC index %d", dac_index);
		return(1);
	}
	
	strcpy(msg, "51xxxxx5");
	msg[MSG_LEN] = '\0';

	/* set two characters in the message we send to STAMP */
	if (dac_index < 8) {
		msg[2] = '0';
		msg[3] = '0';
	} 
	else {
		msg[2] = '1';
		msg[3] = '0';
	}

       	/* get value to load in dac address positions */
       	number = dac_index % 8;
        msg[4] = '0' + 2*number;

       	/* convert the voltage to a DAC setting value */
        this_voltage = voltage;
       	value = 140 - ((this_voltage - .05) / .0974);
	if (value > 255) {
		value = 255;
	}
	if (value < 0) {
		value = 0;
	}
		
	/* convert the DAC setting value into a 2-character code */
	temp3 = floor(value/16);
	temp4 = floor(value - temp3*16);
	msg[5] = '0' + temp3;
	msg[6] = '0' + temp4;

	send_message(msg, reply, "load_one_dac");

	/* change the value stored in the g_voltages[] array */
	g_voltages[dac_index] = voltage;

	return(0);
}


	/*************************************************************
	 * PROCEDURE: show_dacs
	 *
	 * DESCRIPTION:  This routine prints out in a neat, compact form
	 *               the value of all the DACs.  That is, print out
	 *               the number stored in the g_voltages[][]
	 *               array.
	 *
	 * RETURNS:
	 *    nothing
	 */

void
show_dacs
	(
	void
	)
{
	char line[LINELEN+1];
	int i;

	for (i = 0; i < NUM_DACS; i++) {
		sprintf(line, "%2d %14s %7.2f  ",
			i, g_labels[64 + i], g_voltages[i]);
		shLog("%s", line);
	}

}


	/*************************************************************
	 * PROCEDURE: disable_coolers
	 *
	 * DESCRIPTION: Turn off the TEC cooling units attached to
	 *              the CCD cameras.   We do so by setting
	 *              DAC 14 and DAC 15 to the value "9".
	 *
	 * RETURNS:
	 *     nothing
	 */

void
disable_coolers
	(
	void
	)
{
	if (load_one_dac(14, 9) != 0) {
		shError("disable_coolers: load_one_dac fails on DAC 14");
	}
	if (load_one_dac(15, 9) != 0) {
		shError("disable_coolers: load_one_dac fails on DAC 15");
	}
}



	/*************************************************************
	 * PROCEDURE: enable_coolers
	 *
	 * DESCRIPTION: Turn on the TEC cooling units attached to
	 *              the CCD cameras to a _reasonable_ value;
	 *              provide some cooling, but leave room to
	 *              adjust as the ambient temperature drops.
	 *
	 * RETURNS:
	 *     nothing
	 */

void
enable_coolers
	(
	void
	)
{
	if (load_one_dac(14, 6) != 0) {
		shError("enable_coolers: load_one_dac fails on DAC 14");
	}
	if (load_one_dac(15, 6) != 0) {
		shError("enable_coolers: load_one_dac fails on DAC 15");
	}
}






	/*************************************************************
	 * PROCEDURE: read_adcs
	 *
	 * DESCRIPTION:
	 * 
	 *  This routine reads all the Analog-to-Digital-Conversion (ADC) units
	 *    in the STAMP.  It converts their values to the appropriate
	 *    scaled units (in some cases), and places all those values
	 *    into the global array g_adc[][].
	 *
	 *  Note that the global array g_adc[][] has 3 columns.
	 *
	 *              col 0    g_adc[i][0]      is an additive constant
	 *              col 1    g_adc[i][1]      is a multiplicative factor
	 *              col 2    g_adc[i][2]      is the final value
	 *
	 *  We convert the raw value read from the ADC into a final value
	 *  via a 2-step conversion:
	 *
	 *      step 1a: use 4 of the bytes in the "reply" from the STAMP
	 *                  to calculate a "scaled" value
	 *
	 *      step 1b: for a few of the values, add or subtract one
	 *                  of the neighboring ADC values
	 *
	 *      step 2: convert the "scaled" value into a final value 
	 *                  by using the first 2 columns in the g_adc[][]:
	 *
	 *                   (  scaled                  )
	 *          final =  ( --------  +  g_adc[i][0] )  *  g_adc[i][1]
	 *                   (  3276.8                  )
	 *
	 *
	 * RETURNS:
	 *    0             if all goes well
	 *    1             if an error occurs
	 */

int
read_adcs
	(
	void
	)
{
	char msg[MSG_LEN+1], reply[MSG_LEN+1];
	int i;
	int t0, t1, t2, t3;
	double scaled;

	/*
	 * There are NUM_ADCS channels, numbered 0 ... NUM_ADCS-1.  
	 *   We read all of them in turn, and then we "fix" the 
	 *   data returned by some of them.
	 */

	/* Step 1: read all channels */
	strcpy(msg, "520xx005");
	msg[MSG_LEN] = '\0';
	for (i = 0; i < NUM_ADCS; i++) {
		/* send a command to the STAMP: read this channel */
		if (i <= 15) {
			msg[3] = '1';
		} 
		else {
			msg[3] = '2';
		}
		msg[4] = (i % 16) + '0';
		send_message(msg, reply, "read_adcs");

		/* 
		 * now, we scale the result, and place the scaled
		 *   result into element 2 of the g_adc array
		 *   for the element.
		 */
		t3 = reply[3] - '0';
		t2 = reply[4] - '0';
		t1 = reply[5] - '0';
		t0 = reply[6] - '0';
		scaled = 4096*t3 + 256*t2 + 16*t1 + t0;
		if (t3 > 7) {
			scaled -= 65536;
		}

		g_adc[i][2] = ((scaled/3267.8) + g_adc[i][0]) * g_adc[i][1];
	}

	/* 
	 *  Step 2: fixup the data.  We must do this after all the data has
	 *     been read from all the channels.  We CANNOT interleave the
	 *     reading with the fixing!
	 * 
	 *     What exactly is "fixing"?  We remove fixed offsets from
	 *     _some_ of the values, but not all of them.  We just handle
	 *     the special cases which need to be fixed on a case-by-case
	 *     basis here....
	 */
	 for (i = 0; i < NUM_ADCS; i++) {
		 switch (i) {
			case 0:
				 g_adc[0][2] -= (273 + g_adc[3][2]);
				 break;
			case 1:
				 g_adc[1][2] -= (273 + g_adc[3][2]);
				 break;
			case 18:
				 g_adc[18][2] -= (273 + g_adc[16][2]);
				 break;
			case 19:
				 g_adc[19][2] -= (273 + g_adc[16][2]);
				 break;
			case 25:
				 g_adc[25][2] -= g_adc[24][2];
				 break;
			case 26:
				 g_adc[26][2] -= g_adc[24][2];
				 break;
			default:
				 break;
		}

	}

	return(0);
}


	/*************************************************************
	 * PROCEDURE: show_adcs
	 *
	 * DESCRIPTION:  This routine prints out in a neat, compact form
	 *               the value of all the ADCs.  That is, print out
	 *               the number stored in col 2 of the g_adc[][]
	 *               array.
	 *
	 * RETURNS:
	 *    nothing
	 */

void
show_adcs
	(
	void
	)
{
	char line[LINELEN+1];
	int i, other_i;
	int half_num_adcs = NUM_ADCS/2;

	/* we read the ADCs first, of course */
	if (read_adcs() != 0) {
		shError("    show_adcs: read_adcs returns with error");
	}

	for (i = 0; i < half_num_adcs; i++) {
		other_i = i + half_num_adcs;
		sprintf(line, "%2d %14s %7.2f   %2d %14s %7.2f",
			i, g_labels[i], g_adc[i][2], 
			other_i, g_labels[other_i], g_adc[other_i][2]);
		shLog("%s", line);
	}

}

