
	/*
	 * This file contains routines which read the hardware
	 * limit switches on the Mark IV.
	 *
	 * The limit switches are all read simultaneously by calling the
	 * send_serial routine with a message in which  
	 *
	 *        char 1 = "6"     char 2 = "0".  
	 *
	 * (zero-indexed, so char 1 is the second byte in the message).
	 *
	 * The limit switch values are indicated by the values of 
	 * the 5'th and 6'th chars in the reply message.
	 * Specifically,
	 *
	 *       char 5:   byte has bits   xxxx abxx
	 *                 where
	 *                        x       bit isn't used by limit switch
	 *                        a       bit set to 0 if I focus switch light
	 *                                           1 if I focus switch dark
	 *                        b       bit set to 0 if Dec switch light
	 *                                           1 if Dec switch dark
	 *
	 *       char 6:   byte has bits   xxxx xxdc
	 *                 where
	 *                        x       bit isn't used by limit switch
	 *                        d       bit set to 0 if RA switch light
	 *                                           1 if RA switch dark
	 *                        c       bit set to 0 if V focus switch light
	 *                                           1 if V focus switch dark
	 *
	 *
	 * MWR 8/10/2001
	 */

#include <stdio.h>
#include <string.h>
#include "tait.h"

	/****************************************************************
	 * PROCEDURE: dec_is_at_limit
	 *
	 * DESCRIPTION: Check to see if the Dec limit switch is
	 *              triggering.
	 *       
	 *       If telescope is past North Dec limit (elevation > +60 degrees),
	 *                         or South Dec limit (elevation < -90 degrees),
	 *                        then optical beam will reach detector
	 *                        bit 3 of char 5 is set to 0
	 *                            ---> this func returns 1
	 *
	 *       If telescope is not past either Dec limit
	 *                        (that is, elevation is a fine working angle)
	 *                        then optical beam is blocked
	 *                        bit 3 of char 5 arg is set to 1
	 *                            ---> this func returns 0
	 *
	 * RETURNS:
	 *          0          if the telescope is not at either Dec limit
	 *          1          if the telescope is at either of the Dec limits
	 */


int 
dec_is_at_limit
	(
	void
	)
{
	char msg[MSG_LEN+1], reply[MSG_LEN+1];
	int dec_switch;
	int retval = -1;

	/* send a messsage to STAMP, requesting check of limit switches */
	strncpy(msg, "560????5", MSG_LEN);
	msg[MSG_LEN] = '\0';
	send_message(msg, reply, "dec_is_at_limit");

	dec_switch = reply[5] & 0x4;
	if (dec_switch == 0) {
		/* yes, the limit switch is triggering */
		retval = 1;
	}
	else {
		/* no, the Dec limit switch isn't triggering */
		retval = 0;
	}

	return(retval);
}


	/****************************************************************
	 * PROCEDURE: ra_is_at_limit
	 *
	 * DESCRIPTION: Check to see if the RA limit switch is
	 *              triggering.
	 *       
	 *       If telescope is pointing as far East as possible
	 *                        then optical beam will be blocked 
	 *                        bit 2 of char 6 is set to 1
	 *                            ---> this func returns 1
	 *
	 *       If telescope is pointing anywhere else
	 *                        (that is, RA is not at limit)
	 *                        then optical beam reaches detector
	 *                        bit 2 of char 6 arg is set to 0
	 *                            ---> this func returns 0
	 *
	 * RETURNS:
	 *          0          if the telescope is not at RA limit
	 *          1          if the telescope is at RA limit
	 */


int 
ra_is_at_limit
	(
	void
	)
{
	char msg[MSG_LEN+1], reply[MSG_LEN+1];
	int ra_switch;
	int retval = -1;

	/* send a messsage to STAMP, requesting check of limit switches */
	strncpy(msg, "560????5", MSG_LEN);
	msg[MSG_LEN] = '\0';
	send_message(msg, reply, "ra_is_at_limit");

	ra_switch = reply[6] & 0x2;
	if (ra_switch == 0) {
		/* no, the RA limit switch beam reaches the detector */
		retval = 0;
	}
	else {
		/* yes, the RA limit switch beam is blocked */
		retval = 1;
	}

	return(retval);
}


	/****************************************************************
	 * PROCEDURE: foc_is_at_limit
	 *
	 * DESCRIPTION: Check to see if the given camera's focus limit 
	 *              switch is triggering.
	 *
	 * 
	 *        If I focus assembly is pressed close to telescope
	 *                         then optical beam is blocked
	 *                         bit 4 of "f" arg is set to 1
	 *                             ---> this func returns 1
	 * 
	 *        If I focus assembly is moved back away from telescope
	 *                         then optical beam reaches detector
	 *                         bit 4 of "f" arg is set to 0
	 *                             ---> this func returns 0
	 * 
	 *        If V focus assembly is pressed close to telescope
	 *                         then optical beam is blocked
	 *                         bit 1 of "g" arg is set to 1
	 *                             ---> this func returns 1
	 * 
	 *        If V focus assembly is moved back away from telescope
	 *                         then optical beam reaches detector
	 *                         bit 1 of "g" arg is set to 0
	 *                             ---> this func returns 0
	 *
	 *
	 * RETURNS:
	 *          0          if the given camera not at inner focus limit
	 *          1          if the given camera is at inner focus limit
	 */


int 
foc_is_at_limit
	(
	MOTOR_ID id         /* I: which camera's limit switch do we check? */
	)
{
	char msg[MSG_LEN+1], reply[MSG_LEN+1];
	int foc_switch = -1;
	int retval = -1;

	/* sanity check */
	shAssert((id == MOTOR_FOCUSV) || (id == MOTOR_FOCUSI));

	/* send a messsage to STAMP, requesting check of limit switches */
	strncpy(msg, "560????5", MSG_LEN);
	msg[MSG_LEN] = '\0';
	send_message(msg, reply, "foc_is_at_limit");

	if (id == MOTOR_FOCUSI) {
		foc_switch = reply[5] & 0x8;
	} 
	else if (id == MOTOR_FOCUSV) {
		foc_switch = reply[6] & 0x1;
	}
	else {
		shError("    foc_is_at_limit: invalid motor id %d", id);
		return(1);
	}

	if (foc_switch == 0) {
		/* no, the limit switch isn't triggering */
		retval = 0;
	}
	else {
		/* yes, the limit switch is triggering */
		retval = 1;
	}

	return(retval);
}


