#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#include <syslog.h>

#include "llog.h"


#define NO_TIMESTAMP    0          
#define UTC_TIMESTAMP   1
#define LOCAL_TIMESTAMP 2
static int      use_timestamp   = NO_TIMESTAMP;
 
static int      logLevel        = LL_WARNING;
static char     prefix[100]     = "";
static FILE    *log_file        = NULL;
static int      use_syslog      = 0;

/* Forward declaration of localfunctions */
static inline char *get_timestamp();
static inline void VLlog_all(int level, const char *fmt, va_list ap);


/* Set the "level".  Only messages above this will be processed. */
void Llog_setlvl(int level)
{
    if ((level < 0) || (level > LL_MAX_LEVEL))
    {
        /* This function was called with an level set out of range  */
        /* We will reset it to LL_MAX_LEVEL                            */
        logLevel = LL_MAX_LEVEL;
        Llog(LL_WARNING, "log_setlvl called with level = %d resetting to %d\n",
                level, logLevel);
    }
    else
    {
        logLevel = level;
    }
    
    return;
}

/* Set text string to be prepended to ever message.  Normally one would */
/* do something like:   Llog_setprefix("ProgramName: ")                 */
void Llog_setprefix(const char *pre)
{
    if(pre)
    {
        /* pre was specified.  Save it. */
        strncpy(prefix, pre, sizeof(prefix));                                     
    }
    else
    {
        /* null value.  set prefix to null string */
        prefix[0] = '\0';
    }
    
    if(use_syslog)
    {
        closelog();
        openlog(prefix, 0,  LOG_USER);
    }
    return;
}


/* Calling this function will cause all messages to be "timestamped"    */
/* using UTC time.  UTC time is also called GMT time or Zulu time.      */
/* Messages are NOT timestamped if they are being routed to the syslog  */
/* as the syslog facility does it's own timestamping.                   */
void Llog_timestamp_UTC()
{
    use_timestamp = UTC_TIMESTAMP;
    return;
}


/* Calling this function will cause all messages to be "timestamped"    */
/* using the local time base on the user's current time zone            */
/* Messages are NOT timestamped if they are being routed to the syslog  */
/* as the syslog facility does it's own timestamping.                   */
void Llog_timestamp_local()
{
    use_timestamp = LOCAL_TIMESTAMP;
    return;
}


/* Calling this function will cause all messages to be routed to the    */
/* syslog facility and NOT and any other place.                         */
/* When messages are being send to the syslog they are not timestamped  */
/* as the syslog facility does it's own timestamping.                   */
void Llog_syslog()
{
    use_syslog = 1;
    
    openlog(prefix, 0,  LOG_USER);
    return;
}


/* If a valid file name is given all messages will be sent there and    */
/* NOT send to std[err/out].  Note that Llog_syslog() will override.    */
void Llog_setlogfile(const char *logfilename)
{
    if( !logfilename || (0==strlen(logfilename)) )
    {
        if(log_file) fclose(log_file);
        log_file = NULL;
    }
    else
    {   /* Try and open log file.  If it fails leave "log_file" == NULL */
        log_file = fopen(logfilename, "a");
        if(!log_file)
        {
            Llog(LL_WARNING,
                "Could not open log file '%s'.  Using stderr/stdout.\n",
                logfilename);
        }
    }
    return;
}


/* Close the log.  This is not really needed as they will be closed     */
/* automatically when the program terminates but if loging is to be     */
/* stopped or changed before termination this function may help free    */
/* upresources.                                                         */
void Llog_close()
{
    if(use_syslog)
    {
        closelog();
        use_syslog = 0;
    }
    
    if(log_file)
    {
        fclose(log_file);
        log_file = NULL;
    }
    
    /* reset defaults */
    logLevel    = LL_WARNING;
    prefix[0]   = '\0';
    use_timestamp   = NO_TIMESTAMP;
}


/* Process a message if its level is above the current threshold.       */
/* The message may go to either a log file, std[err|out] or to the      */
/* syslog facility.                                                     */
void Llog(int level, const char *fmt, ...)
{
    va_list ap;
    
    if( (level <= logLevel) && fmt )
    {
        /* Validate level */
        if(level < LL_ERROR)          level = LL_ERROR;
        else if(level > LL_MAX_LEVEL) level = LL_MAX_LEVEL;
        
        va_start(ap, fmt);
        VLlog_all(level, fmt, ap);
    }
}

void Llog_debug(const char *fmt, ...)
{
    va_list ap;
    
    if( (LL_DEBUG <= logLevel) && fmt )
    {
        va_start(ap, fmt);
        VLlog_all(LL_DEBUG, fmt, ap);
    }
}

void Llog_verbose(const char *fmt, ...)
{
    va_list ap;
    
    if( (LL_VERBOSE <= logLevel) && fmt )
    {
        va_start(ap, fmt);
        VLlog_all(LL_VERBOSE, fmt, ap);
    }
}


void Llog_warning(const char *fmt, ...)
{
    va_list ap;
    
    if( (LL_WARNING <= logLevel) && fmt )
    {
        va_start(ap, fmt);
        VLlog_all(LL_WARNING, fmt, ap);
    }
}


void Llog_error(const char *fmt, ...)
{
    va_list ap;
    
    if( (LL_ERROR <= logLevel) && fmt )
    {
        va_start(ap, fmt);
        VLlog_all(LL_ERROR, fmt, ap);
    }
}


/************************************************************************/
/************************************************************************/
/**                   Private functions follow                         **/
/************************************************************************/
/************************************************************************/



static inline void VLlog_all(int level, const char *fmt, va_list ap)
{
                           /* LL_ERROR LL_WARNING   LL_VERBOSE  LL_DEBUG */
    static int  sys_lvl[] = { LOG_ERR, LOG_WARNING, LOG_INFO,   LOG_DEBUG };
    char        buff[LLOG_MAXMSG];
    FILE       *file;
    
    /* Clear out the buffer */
    buff[0]='\0';
    
    /* If we are writing to a file we prepend the prefix        */
    /* If we are wring to the syslog then the system will do it */
    /* for us                                                   */
    if(!use_syslog)
    {
        if(prefix[0])
        {
            strncat(buff, prefix, sizeof(buff));
            strncat(buff, "  ",   sizeof(buff));
        }
    }

    /* Only LL_ERROR and LL_WARNING messages get labled as such */
    if     (LL_WARNING == level) strncat(buff, "WARNING ", sizeof(buff));
    else if(LL_ERROR   == level) strncat(buff, "ERROR ",   sizeof(buff));

    /* If requested, and we are not using syslog, add a time stamp */       
    if ( !use_syslog && (NO_TIMESTAMP != use_timestamp) )
    {
        strncat(buff, get_timestamp(), sizeof(buff));
        strncat(buff, "  ",  sizeof(buff));
    }

    /* At last, append a formated message to the buffer */
    vsnprintf(&buff[strlen(buff)], sizeof(buff), fmt, ap);


    /* Write the buffer out to where ever it goes */
    if(use_syslog)
    {
        /* We will use syslog and NOT a file */
        syslog(sys_lvl[level], buff);
    }
    else
    {
        /* We are to use a file.  Either std[err|out] or a log file     */
        if (log_file) 
        {
            file = log_file;
        }
        else
        {
            if (level <= LL_WARNING)
            {
                file=stderr;
            }
            else
            {
                file=stdout;
            }

        }

        fprintf(file, buff); 
        fdatasync(fileno(file));  
    }  
}

static inline char *get_timestamp()
{
    time_t          current_time;
    struct tm      *tmsptr;
    static char     time_str[32];
    int             i;
    
    current_time = time((time_t *)NULL);
    
    if(UTC_TIMESTAMP == use_timestamp)
    {
        tmsptr = gmtime(&current_time);
        strncpy(time_str, asctime(tmsptr), sizeof(time_str));
    }
    else if (LOCAL_TIMESTAMP == use_timestamp)
    {
        tmsptr = localtime(&current_time);
        strncpy(time_str, asctime(tmsptr), sizeof(time_str));
    }
    else
    {
        /* This should never happen. */
        time_str[0] = '\0';
    }  
    
    /* Remove new line from end of string*/ 
    for(i=0; time_str[i]; i++)
    {
        if('\n'== time_str[i])
        {
            time_str[i] = '\0';
        }
    }   
    
    return time_str; 
}
