
#ifndef LLOG_H
#define LLOG_H

#include <stdarg.h>


/* This is the longest message that can be printed.  The size is measured   */
/* after adding any prefix, timestamp and expanding the format              */
/* specification.  If messages are being trucated then you may want to      */
/* either write shorted messages (good idea) or make this constant larger   */
#define LLOG_MAXMSG 4096


/*****************************************************************************/
/**     Use these to specify both the "level" of a message, and to set      **/
/**     the thrshold for logging messages.  For example if the threshold    **/
/**     level is set to "LL_VERBOSE" any message tagged as "LL_DEBUG" will  **/
/**     not be logged.                                                      **/
/*****************************************************************************/
#define LL_SILENT     -1
#define LL_ERROR       0
#define LL_WARNING     1
#define LL_VERBOSE     2
#define LL_DEBUG       3
#define LL_MAX_LEVEL   LL_DEBUG



/*****************************************************************************/
/**                     Define the LLOG_MARK macro                          **/
/**                                                                         **/
/**     To use this macro simply put a line like this in your code:         **/
/**     LLOG_MARK                                                           **/
/**     The above will cause a message containing the source code file and  **/
/**     line number to the logged to where ever you have directed the       **/
/**     mesages to be sent.                                                 **/
/**                                                                         **/
/**     This is a simpleway to trace your program's execution path.  The    **/
/**     log entries will only appear if you have the level                  **/
/**     (see Llog_setlvl()) set to LL_DEBUG.  Also if LLOG_NODEBUG is       **/
/**     defined then this macro does nothing.                               **/
/*****************************************************************************/                           
#ifdef  LLOG_NODEBUG  
#define LLOG_MARK
#else                    
#define LLOG_MARK           Llog(LL_DEBUG, "    %s:%d\n",    __FILE__, __LINE__);
#endif


/*****************************************************************************/
/**                     Define the LLOG_MARK macro                          **/
/**                                                                         **/
/**     To use this macro simply put a line like this in your code:         **/ 
/**     LLOG_DEBUG("Hello World")                                           **/
/**     The above works just like LLOG_MARK except you can supply a simple  **/
/**     comment.                                                            **/
/*****************************************************************************/
#ifdef  LLOG_NODEBUG  
#define LLOG_DEBUG(a)
#else                    
#define LLOG_DEBUG(a)       Llog(LL_DEBUG, "%s  %s:%d\n", a, __FILE__, __LINE__);
#endif  


/*****************************************************************************/
/**                    Define the LLOG_ASSERT macro                         **/
/**                                                                         **/
/**     To use this macro simply put a line like this in your code:         **/
/**     LLOG_ASSERT( 3==4 )                                                 **/
/**     The above will cause a message to be logged to where ever you have  **/
/**     directed the meesages to be sent.  If LLOG_NODEBUG is defined then  **/
/**     the LLOG_ASSERT macro will no be included in your program.          **/
/**                                                                         **/
/**     Use this macro to find programming bugs, not "normal" errors that   **/
/**     could possably occur in a bug free program such as say a file not   **/
/**     being found.                                                        **/
/*****************************************************************************/
#ifdef  LLOG_NODEBUG 
#define LLOG_ASSERT(a) 
#else
#define LLOG_ASSERT(a)      if( !(a) ) { \
Llog(LL_ERROR, "\n>>> LLOG_ASSERT FAILED <<<\n" \
                 "    Assertion:   %s\n" \
                 "    Detected at: %s:%d\n", #a, __FILE__, __LINE__);} 
#endif  


/*****************************************************************************/
/**     These are the user callable functions.  Read usage comments above   **/
/**     function definition in the "dot.c" file.  But for many programs     **/
/**     You may simply folow this example and ignore most of the functions  **/
/**                                                                         **/
/**     #include "llog.h"                                                   **/
/**     int main()                                                          **/
/**     {                                                                   **/
/**         Llog_setlvl(LL_DEBUG);                                          **/
/**         Llog_debug("Hello World, I can count %d, %d, %d\n", 1, 2, 3);   **/
/**         exit(0);                                                        **/
/**     }                                                                   **/
/*****************************************************************************/
extern void Llog_setlvl(int level);
extern void Llog_setprefix(const char *pre);
extern void Llog_timestamp_UTC();
extern void Llog_timestamp_local();
extern void Llog_syslog();
extern void Llog_setlogfile(const char *logfilename);
extern void Llog_close();
extern void Llog(int level, const char *fmt, ...);
extern void Llog_debug     (const char *fmt, ...);
extern void Llog_verbose   (const char *fmt, ...);
extern void Llog_warning   (const char *fmt, ...);
extern void Llog_error     (const char *fmt, ...);


#endif  /* LLOG_H */
