
   /*
    * little support functions 
    *
    */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>
#include "tait.h"



   /*********************************************************************
    * ROUTINE: shMalloc
    *
    * Attempt to allocate the given number of bytes.  Return the 
    * memory, if we succeeded, or print an error message and
    * exit with error code if we failed.
    * 
    * RETURNS:
    *      void *             to new memory, if we got it
    */

void *
shMalloc
   (
   int nbytes                /* I: allocate a chunk of this many bytes */
   )
{
   void *vptr;

   if ((vptr = (void *) malloc(nbytes)) == NULL) {
      shError("shMalloc: failed to allocate for %d bytes", nbytes);
      exit(1);
   }
   return(vptr);
}


   /*********************************************************************
    * ROUTINE: shFree
    *
    * Attempt to free the given piece of memory.  
    * 
    * RETURNS:
    *      nothing
    */

void 
shFree
   (
   void *vptr                /* I: free this chunk of memory */
   )
{
   free(vptr);
}



   /*********************************************************************
    * ROUTINE: shError
    *
    * Print the given error message to stderr, but continue to execute.
    * 
    * RETURNS:
    *      nothing
    */

void 
shError
   (
   char *format,             /* I: format part of printf statement */
   ...                       /* I: optional arguments to printf */
   )
{
   va_list ap;
   time_t current_time;
   struct tm *tms;
   char timestr[100];
   FILE *log_fp;

   /* get the current time */
   current_time = time((time_t *)NULL);
   tms = gmtime(&current_time);
   sprintf(timestr, "%02d/%02d/%04d %02d:%02d:%02d",
		   tms->tm_mon + 1, tms->tm_mday, tms->tm_year + 1900, 
		   tms->tm_hour, tms->tm_min, tms->tm_sec);
	
   va_start(ap, format);

   /* print to stderr */
   fprintf(stderr, "%s ERROR ", timestr);
   (void) vfprintf(stderr, (const char *)format, ap);
   fputc('\n', stderr);
   fflush(stdout);
   fflush(stderr);

   /* append to the log file */
   if ((log_fp = fopen(LOGFILE, "a")) == NULL) {
	   shFatal("shError: can't open LOGFILE %s", LOGFILE);
   }
   fprintf(log_fp, "%s ERROR", timestr);
   (void) vfprintf(log_fp, (const char *)format, ap);
   fputc('\n', log_fp);
   fflush(log_fp);
   fclose(log_fp);

   va_end(ap);

}


   /*********************************************************************
    * ROUTINE: shFatal
    *
    * Print the given error message to stderr, and halt program execution.
    * 
    * RETURNS:
    *      nothing
    */

void 
shFatal
   (
   char *format,             /* I: format part of printf statement */
   ...                       /* I: optional arguments to printf */
   )
{
   va_list ap;
   time_t current_time;
   struct tm *tms;
   char timestr[100];
   FILE *log_fp;

   /* get the current time */
   current_time = time((time_t *)NULL);
   tms = gmtime(&current_time);
   sprintf(timestr, "%02d/%02d/%04d %02d:%02d:%02d",
		   tms->tm_mon + 1, tms->tm_mday, tms->tm_year + 1900, 
		   tms->tm_hour, tms->tm_min, tms->tm_sec);
	
   va_start(ap, format);

   /* print to stderr */
   fprintf(stderr, "%s FATAL ", timestr);
   (void) vfprintf(stderr, (const char *)format, ap);
   fputc('\n', stderr);
   fflush(stdout);
   fflush(stderr);

   /* append to the log file */
   if ((log_fp = fopen(LOGFILE, "a")) == NULL) {
	   shFatal("shFatal: can't open LOGFILE %s", LOGFILE);
   }
   fprintf(log_fp, "%s FATAL ", timestr);
   (void) vfprintf(log_fp, (const char *)format, ap);
   fputc('\n', log_fp);
   fflush(log_fp);
   fclose(log_fp);

   va_end(ap);
   exit(1);
}


   /*********************************************************************
    * ROUTINE: shLog
    *
    * Print the given message to stdout, and also append it to 
    * the LOGFILE.  Prepend the message with a timestamp of the 
    * form
    *             DD/MM/YYYY HH:MM:SS
    * 
    * RETURNS:
    *      nothing
    */

void 
shLog
   (
   char *format,             /* I: format part of printf statement */
   ...                       /* I: optional arguments to printf */
   )
{
   FILE *log_fp;
   va_list ap;
   time_t current_time;
   struct tm *tms;
   char timestr[100];
   
   /* get the current time */
   current_time = time((time_t *)NULL);
   tms = gmtime(&current_time);
   sprintf(timestr, "%02d/%02d/%04d %02d:%02d:%02d",
		   tms->tm_mon + 1, tms->tm_mday, tms->tm_year + 1900, 
		   tms->tm_hour, tms->tm_min, tms->tm_sec);
	

   /* first, print to stdout */
   va_start(ap, format);
   fprintf(stdout, "%s ", timestr);
   (void) vfprintf(stdout, (const char *)format, ap);
   fputc('\n', stdout);
   fflush(stdout);

   /* now, append to LOGFILE */
   if ((log_fp = fopen(LOGFILE, "a")) == NULL) {
	   shFatal("shLog: can't open LOGFILE %s", LOGFILE);
   }
   fprintf(log_fp, "%s ", timestr);
   (void) vfprintf(log_fp, (const char *)format, ap);
   fputc('\n', log_fp);
   fflush(log_fp);
   fclose(log_fp);

   va_end(ap);
}


