
	/*
	 * code to control motors directly, at a low level 
	 */


#include <stdio.h>
#include <string.h>
#include <math.h>
#include "tait.h"


static int setup_motor(MOTOR_DIRECTION direction);
static int select_motor(MOTOR_ID motor, char *message);
static int motor_on(char *message);
static int motors_off(void);
static int set_mask(MOTOR_ID motor_id, char *message);
static int move_nsteps(int nsteps, char *message);
static int move_to_limit(char *message);



	/***********************************************************************
	 * PROCEDURE: motor
	 *
	 * DESCRIPTION: send signals directly to one of the motors.
	 *
	 * RETURNS:
	 *    0         if all goes well
	 *    1         if not
	 */

int
motor
	(
	MOTOR_ACTION action,        /* I: do what to motor? */
	MOTOR_ID motor_id,          /* I: which motor? */
	MOTOR_DIRECTION direction,  /* I: if action is to move in a direction */
	                            /*      which direction is it? */
	int steps                   /* I: if action is to move in a direction */
	                            /*      how many steps should we move? */
	)
{
	MOTOR_DIRECTION dir;
	char message[MSG_LEN + 1];

#ifdef DEBUG
	printf("entering motor \n");
#endif

	strncpy(message, "00000000", MSG_LEN);
	message[MSG_LEN] = '\0';

	switch (action) {
	case MOTOR_LIMIT_OUT:
		/* we start by moving in towards the limit switch */
		dir = MOTOR_TOWARD_LIMIT;
		setup_motor(dir);
		select_motor(motor_id, message);
		motor_on(message);
		set_mask(motor_id, message);
		move_to_limit(message);

		/* and now we move back out by the requested number of steps */
		dir = MOTOR_FROM_LIMIT;
		setup_motor(dir);
		move_nsteps(steps, message);
		motors_off();
		break;

	case MOTOR_IN_DIRECTION:
#ifdef DEBUG2
{ 
#define GETS 0
	char line[200];
#endif
		dir = direction;
#ifdef DEBUG2
shLog("about to call setup_motor with dir %d ", dir);
#if GETS
gets(line);
#endif
#endif
		setup_motor(dir);
#ifdef DEBUG2
shLog("about to call select_motor with motor_id %d msg %s ", 
motor_id, message);
#if GETS
gets(line);
#endif
#endif
		select_motor(motor_id, message);
#ifdef DEBUG2
shLog("about to call motor_on with msg %s ", 
message);
#if GETS
gets(line);
#endif
#endif
		motor_on(message);
#ifdef DEBUG2
shLog("about to call move_nsteps with nsteps %d msg %s ", 
steps, message);
#if GETS
gets(line);
#endif
#endif
		move_nsteps(steps, message);
#ifdef DEBUG2
shLog("about to call motor_off with msg %s ", 
message);
#if GETS
gets(line);
#endif
#endif
		motors_off();
#ifdef DEBUG2
shLog("all done with motor");
}
#endif

		break;

	case MOTOR_OFF:
		motors_off();
		break;

	default:
		shError("motor: invalid action %d ", action);
		return(1);
		break;
	}

	return(0);
}



	/***********************************************************************
	 * PROCEDURE: setup_motor
	 *
	 * DESCRIPTION: Load the direction (and last position?) for the
	 *              given motor.  The motor can remember where it is,
	 *              but we need to save that information in a file 
	 *              between power on/off.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
setup_motor
	(
	MOTOR_DIRECTION direction          /* I: direction to move motor */
	)
{
	char buf[MSG_LEN + 1];
	char reply[MSG_LEN + 1];

	shAssert((direction == MOTOR_LIMIT_OUT) || 
	         (direction == MOTOR_IN_DIRECTION) ||
	         (direction == MOTOR_OFF));

	buf[0] = '0';
	buf[1] = '9';
	buf[2] = '4';
	buf[3] = '0' + direction;
	buf[4] = '0';
	buf[5] = '0';
	buf[6] = '0';
	buf[7] = '0';
	buf[MSG_LEN] = '\0';

	send_message(buf, reply, "setup_motor");

	return(0);
}


	/***********************************************************************
	 * PROCEDURE: select_motor
	 *
	 * DESCRIPTION: Set two bytes (numbers 5 and 6) in the given 
	 *              'message' string, depending on the given motor.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
select_motor
	(
	MOTOR_ID motor,         /* I: which motor will we control? */
	char *message      /* O: an 8-byte message in which we set two bytes */
	)
{

	shAssert((motor == MOTOR_FOCUSV) || 
	         (motor == MOTOR_DECDRIVE) ||
	         (motor == MOTOR_DECCLAMP) ||
	         (motor == MOTOR_FOCUSI));
	shAssert(message != NULL);

	switch (motor) {
	case MOTOR_FOCUSV:
		message[5] = '7';
		message[6] = '?';
		break;
	case MOTOR_DECDRIVE:
		/* a value of '9' in byte 5 would turn on both DECDRIVE and DECCLAMP */
		message[5] = ';';
		message[6] = '?';
		break;
	case MOTOR_DECCLAMP:
		message[5] = '=';
		message[6] = '?';
		break;
	case MOTOR_FOCUSI:
		message[5] = '>';
		message[6] = '?';
		break;
	default:
		shError("select_motor: invalid motor number %d", motor);
		return(1);
		break;
	}

	return(0);
}


	/***********************************************************************
	 * PROCEDURE: motor_on
	 *
	 * DESCRIPTION: Sends a message to the Stamp to turn on the motor
	 *              which is indicated by the bytes 5 and 6 of the
	 *              given message string (which must already have
	 *              been set).
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
motor_on
	(
	char *message          /* I/O: a message for the Stamp */
	                       /*      which must already have bytes 5,6 set */
	)
{
	char reply[MSG_LEN + 1];

	shAssert(message != NULL);
	
	message[1] = '8';         /* test */
	message[2] = '1';         /* load register */
	message[3] = '5';         /* motor enable register */
	message[4] = '9';

	send_message(message, reply, "motor_on");

	return(0);
}



	/***********************************************************************
	 * PROCEDURE: motors_off
	 *
	 * DESCRIPTION: Sends a message to the Stamp to turn off all
	 *              the motors.
	 *
	 *              Current method means that 
	 *              the Dec motor is always on.  Should be ok as the 
	 *              current draw is low.  There is the possibility 
	 *              of noise from a motor running,
	 *              but I doubt it as the RA drive motor is 
	 *              running all the time too.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
motors_off
	(
	void
	)
{
	char message[MSG_LEN + 1], reply[MSG_LEN + 1];

	message[0] = '0';
	message[1] = '8';         /* test */
	message[2] = '1';         /* load register */
	message[3] = '5';         /* motor enable register */
	message[4] = '9';
	message[5] = '?';         /* all high = motors off */
	message[6] = '?';         /* all high = motors off */
	message[7] = '0';
	message[MSG_LEN] = '\0';

	send_message(message, reply, "motor_on");

	return(0);
}



	/***********************************************************************
	 * PROCEDURE: set_mask
	 *
	 * DESCRIPTION: Set bytes 3,4,5,6 in the given 'message' string
	 *              for the stamp, based on the given motor ID.
	 *              This is the "motor limit mask" information.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
set_mask
	(
	MOTOR_ID motor_id,     /* I: which motor are we going to move? */
	char *message          /* O: a message for the Stamp */
	                       /*      in which we'll set bytes 3,4,5,6 */
	)
{

	shAssert(message != NULL);
	shAssert((motor_id == MOTOR_FOCUSV) || 
	         (motor_id == MOTOR_DECDRIVE) ||
	         (motor_id == MOTOR_DECCLAMP) ||
	         (motor_id == MOTOR_FOCUSI));

	switch (motor_id) {
	case MOTOR_FOCUSV:
		message[3] = '0';
		message[4] = '0';
		message[5] = '0';
		message[6] = '1';
		break;
	case MOTOR_DECDRIVE:
		message[3] = '0';
		message[4] = '0';
		message[5] = '4';
		message[6] = '0';
		break;
	case MOTOR_DECCLAMP:
		message[3] = '0';
		message[4] = '0';
		message[5] = '0';
		message[6] = '0';
		break;
	case MOTOR_FOCUSI:
		message[3] = '0';
		message[4] = '0';
		message[5] = '8';
		message[6] = '0';
		break;
	default:
		shError("set_mask: invalid motor ID %d", motor_id);
		return(1);
		break;
	}

	return(0);
}


	/***********************************************************************
	 * PROCEDURE: move_nsteps  
	 *
	 * DESCRIPTION: Sends a message to the Stamp which causes
	 *              a motor to move to by the given number of steps.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
move_nsteps
	(
	int nsteps,            /* I: move this many steps */
	char *message          /* I/O: a message for the Stamp */
	)
{
	int temp1, temp2, temp3, temp4;
	char reply[MSG_LEN + 1];

	shAssert(message != NULL);
	
	message[1] = '9';         
	message[2] = '2';         

	/* 
	 * now, we must translate the number of steps into a special
	 * code the STAMP understands: we use 4 bytes to encode the number
	 * of steps.
	 */
	temp1 = floor(nsteps / 4096.0);
	temp2 = floor((nsteps - temp1 * 4096) / 256.0);
	temp3 = floor((nsteps - temp1 * 4096 - temp2 * 256) / 16.0);
	temp4 = nsteps - temp1 * 4096 - temp2 * 256 - temp3 * 16;
	message[3] = temp1 + 0x30;
	message[4] = temp2 + 0x30;
	message[5] = temp3 + 0x30;
	message[6] = temp4 + 0x30;
#ifdef DEBUG
	printf ("move_nsteps: nsteps %6d  bytes are %c %c %c %c \n",
			nsteps, message[3], message[4], message[5], message[6]);
#endif

	send_message(message, reply, "move_nsteps");

	return(0);
}




	/***********************************************************************
	 * PROCEDURE: move_to_limit
	 *
	 * DESCRIPTION: Sends a message to the Stamp which causes
	 *              a motor to move to its limit switch (and then stop).
	 *
	 *              This command sets bytes 1,2 of the given 'message'
	 *              (which must already have bytes 3,4,5,6 set),
	 *              then sends the message to the STAMP.
	 *
	 * RETURNS:
	 *     0        if all goes well
	 *     1        if not
	 */

static int
move_to_limit
	(
	char *message          /* I/O: a message for the Stamp */
	                       /*      which must already have bytes 3,4,5,6 set */
	)
{
	char reply[MSG_LEN + 1];

	shAssert(message != NULL);
	
	message[1] = '9';         
	message[2] = '3';         

	send_message(message, reply, "move_to_limit");

	return(0);
}



