
	/*
	 * Code to convert a user's requested (RA, Dec) position to the
	 * units the camera understands.
	 *
	 * MWR 9/16/2001
	 */

#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include "tait.h"
#include "radec.h"


	/***********************************************************************
	 * PROCEDURE: goto_radec
	 *
	 * DESCRIPTION: The user provides an (RA, Dec), in decimal degrees.
	 *              We do the following:
	 *
	 *                 1. Check to make sure that it lies within the 
	 *                    acceptable range of motion.  If not, 
	 *                    we return with code 1.
	 *
	 *                 2. Go to Dec home, then the given Dec
	 *
	 *                 3. Go to RA home, then the given RA
	 *
	 * RETURNS:
	 *    RADEC_OK       if all goes well
	 *    RADEC_INVALID  if the given (RA, Dec) position in not currently valid
	 *    RADEC_ERROR    if some error occurred
	 */

extern int dec_move_abs(int steps);
extern int cmd_ra(char *cmdline);

int 
goto_radec
	(
	double ra,              /* I: desired RA, in decimal degrees */
	double dec              /* I: desired Dec, in decimal degrees */
	)
{
	int steps;
	int seconds;
	double jd;
	double lst;
	double ha;

	/* 
	 * step 1: calculate the Hour Angle for the given RA --
	 *   we need to get the LST to do this, and then calculate
	 *                HA = LST - RA
	 *
	 *   We want to end up with HA in decimal degrees, West of meridian
	 */
	get_now_jd(&jd);
	get_lst(jd, &lst);
	/* get_lst yields an LST in decimal hours; we must convert to degrees */
	lst *= 15.0;
	ha = lst - ra;

	/* make sure that the HA lies between -180 and 180 degrees */
	while (ha > 180) {
		ha -= 180.0;
	}
	while (ha < -180) {
		ha += 180.0;
	}

	
	/* step 2: check that the given (RA, Dec) in within valid ranges */
	if ((ha < HA_LIMIT_MIN) || (ha > HA_LIMIT_MAX)) {
		shError("ha %9.4f is outside permitted range", ha);
		return(RADEC_INVALID);
	}
	if ((dec < DEC_LIMIT_MIN) || (dec > DEC_LIMIT_MAX)) {
		shError("dec %9.4f is outside permitted range", dec);
		return(RADEC_INVALID);
	}


	/* 
	 * step 3: move to the desired Dec.
	 */
	steps = floor((dec - DEC_OF_HOME)/DEC_STEP_SIZE);
#ifdef DEBUG
	shLog("goto_radec: Dec %7.2f corresponds to %6d steps", dec, steps);
#endif
	if (dec_move_abs(steps) != 0) {
		return(RADEC_ERROR);
	}

	/*
	 * step 4: move to the desired RA (really, the desired Hour Angle)
	 */
	seconds = floor((ha - HA_OF_HOME)/(RA_SLEW_RATE));
#ifdef DEBUG
	shLog("goto_radec: RA %7.2f = HA %7.2f means %6d seconds", ra, ha, seconds);
#endif
	if (cmd_ra("ra off") != PARSE_OK) {
		shError("goto_radec: cmd_ra fails on 'ra off'");
		return(RADEC_ERROR);
	}
	if (cmd_ra("ra tolimit") != PARSE_OK) {
		shError("goto_radec: cmd_ra fails on 'ra tolimit'");
		return(RADEC_ERROR);
	}
	if (cmd_ra("ra rew") != PARSE_OK) {
		shError("goto_radec: cmd_ra fails on 'ra rew'");
		return(RADEC_ERROR);
	}
	/* now wait for the proper number of seconds while RA drive moves */
	sleep(seconds);
	/* stop the RA drive */
	if (cmd_ra("ra off") != PARSE_OK) {
		shError("goto_radec: cmd_ra fails on 'ra off'");
		return(RADEC_ERROR);
	}


	return(RADEC_OK);
}

