
	/*
	 * This file contains routines which talk to the memory card
	 * of the Mark IV.
	 *
	 * The memory card is an ISA card.  We assume that it sits
	 * in address space at MEMCARD_BASE.
	 *
	 * MWR 8/19/2001
	 */

#include <stdio.h>
#include <unistd.h>       /* for usleep */
#include <string.h>
#include <sys/io.h>       /* for port I/O */
#include "tait.h"

static int pulseit(int channel);
static int wait_for_block_done(void);

	/****************************************************************
	 * PROCEDURE: init_memcard
	 *
	 * DESCRIPTION: Tell the kernel that we're going to want to 
	 *              read/write from the ports to which the memory
	 *              card is attached. 
	 *
	 *              We need 2 ports, the MEMCARD_BASE and the address
	 *              immediately above it (i.e. MEMCARD_BASE + 1).
	 *
	 * RETURNS:
	 *          0          if all goes well
	 *          1          if not
	 */


int 
init_memcard
	(
	void
	)
{
	int retval = -1;

	retval = ioperm(MEMCARD_BASE, 2, 1);
	if (retval != 0) {
		shError("    init_memcard: ioperm returns with error");
	}

	return(retval);
}


	/****************************************************************
	 * PROCEDURE: scan_both
	 *
	 * DESCRIPTION: Send signals to start a scan of both CCD 
	 *              cameras.  
	 *
	 * RETURNS:
	 *          0          if the telescope is not at either Dec limit
	 *          1          if the telescope is at either of the Dec limits
	 */


int 
scan_both
	(
	void
	)
{
	int retval = -1;

	/* First, we reset the memory in the memory card */
	outb(0, MEMCARD_BASE);

	/* 
	 * Now we send the signals which reset the "mar",
	 * and then initiate the scan
	 */
	if (pulseit(17) != 0) {
		shError("   scan_both: pulseit(17) returns with error");
		return(1);
	}
	if (pulseit(4) != 0) {
		shError("   scan_both: pulseit(4) returns with error");
		return(1);
	}

#if 1
	retval = wait_for_block_done();
#else
	retval = 0;
#endif
	if (retval != 0) {
		shError("   scan_both: wait_for_block_done returns with error");
	}

	return(retval);
}

	/****************************************************************
	 * PROCEDURE: pulseit
	 * 
	 *  Send a signal to the STAMP which causes a Pulse signal to 
	 *  be sent to the given Channel of the CCD controller(s) 
	 *  (both of them, I think?)
	 * 
	 *  In ordinary use, one reads out (both) CCDs by calling 
	 *  IN THIS ORDER
	 *           PulseIt(17)
	 *           PulseIt(4)
	 * 
	 *  Here are the meanings of the Channels I know:
	 * 
	 *             Channel
	 *                 4              start readout scan
	 *                16              sends "block done" to memory board
	 *                17              reset the "mar" 
	 *                                    (Memory Address Register?)
	 * 
	 *
	 * RETURNS:
	 *          0          if all goes well
	 *          1          if an error occurs
	 */

static int 
pulseit
	(
	int channel            /* send signal to this channel */
	)
{
	char msg[MSG_LEN+1], reply[MSG_LEN+1];
	int retval = -1;

	/* sanity check */
	shAssert((channel >= 0) && (channel <= 23));

	/* set most the characters in the message */
	strncpy(msg, "53000005", MSG_LEN);
	msg[MSG_LEN] = '\0';

	/* 
	 * A brute force conversion of channel to 
	 *       Pulse Select Register contents.
	 */
#define PULSECHARS(a,b) { msg[3] = a; msg[4] = b; }

	switch (channel) {
		case 0:
			PULSECHARS('5', '1');
			break;
		case 1:
			PULSECHARS('7', '1');
			break;
		case 2:
			PULSECHARS('5', '9');
			break;
		case 3:
			PULSECHARS('7', '9');
			break;
		case 4:
			PULSECHARS('5', '3');
			break;
		case 5:
			PULSECHARS('7', '3');
			break;
		case 6:
			PULSECHARS('5', ';');
			break;
		case 7:
			PULSECHARS('7', ';');
		 	break;
		case 8:
			PULSECHARS('9', '1');
			break;
		case 9:
			PULSECHARS(';', '1');
			break;
		case 10:
			PULSECHARS('9', '9');
			break;
		case 11:
			PULSECHARS(';', '9');
			break;
		case 12:
			PULSECHARS('9', '3');
			break;
		case 13:
			PULSECHARS(';', '3');
			break;
		case 14:
			PULSECHARS('9', ';');
			break;
		case 15:
			PULSECHARS(';', ';');
			break;
		case 16:
			PULSECHARS('=', '0');
			break;
		case 17:
			PULSECHARS('?', '0');
			break;
		case 18:
			PULSECHARS('=', '8');
			break;
		case 19:
			PULSECHARS('?', '8');
			break;
		case 20:
			PULSECHARS('=', '2');
			break;
		case 21:
			PULSECHARS('?', '2');
			break;
		case 22:
			PULSECHARS('=', ':');
			break;
		case 23:
			PULSECHARS('?', ':');
			break;
		default:
			shError("pulseit: invalid channel %d", channel);
			return(1);
			break;
		}

#if 1
	retval = send_message(msg, reply, "pulseit");
#else
	printf(" message is ..%s..\n", msg);
	retval = 0;
#endif

	return(retval);
}


	/***************************************************************
	 * PROCEDURE: wait_for_block_done
	 *
	 * DESCRIPTION: Wait until the Memory board gives a signal 
	 *              that the current transfer of information 
	 *              has completed.
	 * 
	 *              However, to prevent an infinite loop, 
	 *              we timeout after "BLOCK_TIMEOUT" seconds if 
	 *              the transfer hasn't finished by then.
	 * 
	 *  RETURNS:
	 *       0             if the Memory board finished the transfer
	 *       1             if the Timeout interrupted the process
	 *                            (which means data transfer failed)
	 */

	/* 
	 * We wait this many seconds for the CCDs to be read out.  
	 * If this many seconds pass and the readout hasn't finished,
	 * we quit waiting and return with an error.
	 */
#define BLOCK_TIMEOUT      60
	/* this should yield a micro-sleep of 0.1 seconds */
#define BLOCK_INIT_WAIT    100000
	/* this should yield a micro-sleep of 0.2 seconds */
#define BLOCK_FINISH_WAIT  200000
	/* this should yield a micro-sleep of 1.0 seconds */
#define BLOCK_ONE_SECOND  1000000

static int
wait_for_block_done
	(
	void
	)
{
	int iter, done;
	int byte;
	int retval;
	
	/*
	 * This apparently is necessary to prevent us from reading
	 *   port H300 before the memory board has managed to send
	 *   a signal to it.
	 */
	usleep(BLOCK_INIT_WAIT);

	/*
	 *  Now, we enter a loop which can terminate one of two ways:
	 * 
	 *  1. the data transfer can finish --> Good
	 *        happens if the byte we read from port H300 has 128-bit set
	 * 
	 *  2. we iterate too many times through this loop and time out --> Bad
	 */
	iter = 0;
	done = 0;
	while (!done) {
		byte = inb(MEMCARD_BASE);
		done = byte & 0x80;

		usleep(BLOCK_ONE_SECOND);
#ifdef DEBUG
		printf("    wait_for_block_done: waiting iter %d", iter);
#endif
		if (++iter > BLOCK_TIMEOUT) {
			done = 1;
		}
	}

	/*
	 * this takes care of the present prom set which puts out 
	 * a block done while a line or two is still reading out
	 */
	usleep(BLOCK_FINISH_WAIT);

	if (iter < BLOCK_TIMEOUT) {
		retval = 0;
	}
	else {
		retval = 1;
	}

	return(retval);
}

