#include <stdlib.h>
#include <string.h>

#include "llog.h"
#include "sd_table.h"


sdent sd_tab[MAX_CLIENTS];


static int client_count = 0;

static int get_index(int sd);
static void remove_ent(int index);

void sd_init_table()
{
    /* Handle condition where the table has been in use */
    while(client_count)
    {
        remove_ent(client_count-1);   
    }
       
    return;
}

int sd_new_sd(int sd)
{
    char *ptr;
    
    /* Check if the table is full */
    if(client_count >= MAX_CLIENTS)
    {
        /* Table is full */
        LLOG_DEBUG("sd_tab is FULL\n");
        return -1;
    }
    
        
    if( !(ptr = malloc(BUFFER_SIZE)) )
    {
        Llog_warning("out of memory, malloc failed\n");
        LLOG_MARK;
        return -1;
    }
    memset(ptr, 0, BUFFER_SIZE);
    
    sd_tab[client_count].sd         = sd;
    sd_tab[client_count].buff       = ptr;
    sd_tab[client_count].buff_ptr   = ptr;
    client_count++;
    
    
    return 0;
}

int sd_remove_sd(int sd)
{
    int index;
    
    if( -1 == (index = get_index(sd)))
    {
        /* ERROR attempt to remove entry not in table */
        LLOG_DEBUG("attempt to remove entry not in sd_tab\n");
        return -1; 
    } 
    
    /* remove entry from table */
    remove_ent(index);
    return 0;
}

int sd_remove_line(sdent *sd_ptr, char *dest, int size)
{
    char *newline;
    int   len;
    
    if( !sd_ptr)
    {
        /* ERROR */
        Llog_warning("sd_remove_line passed NULL sd_ptr\n");
        return -1; 
    } 
    
    
    
    newline = index(sd_ptr->buff, '\n');
    if( !newline )
    {
        /* no full line in buffer, not an error. */
        dest[0] = '\0';
        return 0;
    }
    
    len = 1 + (newline - sd_ptr->buff);
    Llog_debug(" remove a line where len=%d\n", len);

    /* Check that the line we got is not longer than the requested  */
    /* line lenght                                                  */
    if(len >= size)
    {
        /* line is to long */
        Llog_warning("Line recv'd from sock %d is to long\n",sd_ptr->sd);

        /* reset buffer */
        sd_ptr->buff_ptr = sd_ptr->buff;
        memset(sd_ptr->buff_ptr, 0, BUFFER_SIZE);
        dest[0] = '\0';
        return -1;
    }

    /* Now we will move the line into the user's "dest" buffer and  */
    /* add a NULL terminator.                                       */
    memcpy(dest, sd_ptr->buff, len);
    dest[len] = '\0';
    
    /* If the line contains the DOS end of line convention, \r\n    */
    /* we will convert it to the UNIX convention of just a newline  */
    /* character \n.                                                */
    if( (len > 1)               && 
        ('\r' == dest[len-2])   &&
        ('\n' == dest[len-1]) )
    {
        dest[len-2] = '\n';
        dest[len-1] = '\0';
        LLOG_DEBUG("converted DOS line terminator to UNIX convention\n"); 
    }

    /* left justify and zero pad remaing data in buffer */
    memmove( sd_ptr->buff,       &sd_ptr->buff[len], BUFFER_SIZE-len);
    memset (&sd_ptr->buff[len],  0,                       BUFFER_SIZE-len);
    
    /* Remove a leading NULL character if present */
    if( !(sd_ptr->buff[0]) )
    {
        memmove( &sd_ptr->buff[0], &sd_ptr->buff[1], BUFFER_SIZE-1);
    }
    
    
    sd_ptr->buff_ptr = &sd_ptr->buff[strlen(sd_ptr->buff)];
    return strlen(dest);
} 




sdent *sd_get_sdent(int sd)
{
    int index;
    
    if( -1 == (index = get_index(sd)))
    {
        /* Not found in table */
        LLOG_DEBUG("sd_get_sdent sd not in sd_tab\n");
        return NULL;
    }
           
    return &sd_tab[index];
}

static int get_index(int sd)
{
    int i;
    
    
    for(i=0; i<client_count; i++)
    {
        if(sd == sd_tab[i].sd) return i;
    }
    
    /* not found */
    Llog_debug("sd_get_sdent, %d not in sd_tab\n", sd);
    return -1;
}

static void remove_ent(int index)
{
    int i;  
    
    /* Free the buffer */
    if(sd_tab[index].buff)
    {
        free(sd_tab[index].buff);        
    }
    
    /* compress the table */
    for(i=index+1; i<client_count; i++)
    {
        sd_tab[i-1] = sd_tab[i];
    }
    
    client_count--;
    return;
}

/* Main program for testing */
#if 0

#include <stdio.h>


int sd_append2buff(int sd, const char *text)
{
    int index;
    int len;
    int remaing_space;
        
 
    if( -1 == (index = get_index(sd)))
    {
        /* ERROR attempt append entry not in table */
        LLOG_DEBUG("attempt append entry not in sd_tab\n");
        return -1; 
    } 
    
    remaing_space =
        BUFFER_SIZE - (1+sd_tab[index].buff-sd_tab[index].buff_ptr);
   
    /* if no text supplied then do nothing */
    if(!text)
    {
        LLOG_DEBUG("sd_append2buff with NULL text\n");
        return 0;
    }
    
 
    /* prevent buffer overflow */
    len = strlen(text);
    if(remaing_space < len)
    {
        len = remaing_space;
    }
    
    strncpy(sd_tab[index].buff_ptr, text, len);
    sd_tab[index].buff_ptr += len;
 
    return len;
}

void print_tab()
{
    int i;
    
    printf("Table ontains %d entries\n", client_count);
    
    for(i=0; i<client_count; i++)
    {
        printf("  index %d,  sd %d, buff %x,  buff_ptr %x, *buf'%s'\n",
            i, sd_tab[i].sd, (int)sd_tab[i].buff, (int)sd_tab[i].buff_ptr, sd_tab[i].buff);
    }
    
    printf("\n");
}

int main()
{
    sdent *sdent_ptr;
    
    char   test1[] = "This is a test";
    char   line[100];
    
    sd_init_table();
    
    sd_new_sd(3);
    sd_new_sd(4);
    sd_new_sd(5);
    print_tab();
    
    
    sd_remove_sd(4);
    print_tab();
    
    sdent_ptr = sd_get_sdent(3);
    strcpy( sdent_ptr->buff, test1 ); 
    sdent_ptr->buff_ptr += strlen(test1);

    sd_append2buff(5, "this goes with sd5\n");
    print_tab();
    
    sd_remove_line(5, line, sizeof(line));
    print_tab();
    
    sd_append2buff(3, "end of line\nbeginnig of next");
    print_tab();
    sd_remove_line(3, line, sizeof(line));
    print_tab();
    
    exit(0);
}

#endif
