#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>


#include "llog.h"
#include "libsl.h"
#include "sd_table.h"


int sl_Setup_Server_Socket(int port, char *service);
int sl_Wait4Client(int socket);
int sl_SendBuff(int sock, char *buff, int size);
int sl_GetBuff(int sock, char *buff, int size);
int sl_GetLine(int sock, char *dest, int size);

int remove_line(char *dest, int size);
int validate_args(int sock, char *dest, int size);

/************************************************************************/
/*									*/
/*  sl_Setup_Server_Socket						*/
/*									*/
/*  Returns an "Socket Descripter" or "SD" that is and is being         */
/*  The SD will be in "listen" mode.  On error return 0.                */
/*                                                                      */
/*  Input Parameters:                                                   */
/*      port -  If a non-zero port is given that port number will       */
/*              be used.  If port==0 then "service" will be used.       */
/*                                                                      */
/*      service - The name of a TCP service.  as listed in the file     */
/*              /etc/services or in the NIS "services" map.             */
/*              If port==0 then we will try to look uop the service     */
/*              name o get the port number.                             */
/*                                                                      */
/*      If both port==0 and service==NULL then a default port number    */
/*      is used.  Thisis likely only good for development testing       */
/*      as the portnumber should not be used in the "real world".       */
/************************************************************************/
int sl_Setup_Server_Socket(int port, char *service)
{

        int			sd;
        struct   sockaddr_in	sin;
        struct   servent       *my_service;
        int                     retries;
        int                     sleep_time;


        /* Create an internet domain socket */
        if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
        {
            /* Unable to create a socket. */
            Llog_error("ERROR Unable to create a socket: %s\n", 
                strerror(errno));
            return(-1);
        }


        /* Fill in the socket name structure */
        memset(&sin, 0, sizeof(sin));
        sin.sin_family = AF_INET;
        sin.sin_addr.s_addr = INADDR_ANY;


        /* Get a port number for the server.			        */
        /* If -p or --port argument was specified use that else look	*/  
        /* in the /etc/services file.					*/
        if (port > 0)
        {
            /* port was given.  Use it. */
            sin.sin_port = htons( (short int)port );
            Llog_debug("Given port number\n");
        }
        else if (service)
        {
            if ((my_service = getservbyname(service, "tcp")) == NULL)
            {
        	/* Failed to find the tass service.  Maybe the */
        	/* /etc/services file does not have a tass     */
        	/* entry in it?                                */
        	Llog_warning("Failed to find service '%s'\n", service);
        	return(-1);
            }
            sin.sin_port = my_service->s_port;
            Llog_debug("Got port number from /etc/services file\n");
        }
        else
        {
            Llog_debug("using default port\n");
            sin.sin_port = htons( (short int)DEFAULT_PORT );
        }
        
        Llog_verbose("Using port %d \n", ntohs(sin.sin_port));

        /* bind the socket to the port number, try a few times */
        retries    = 5;
        sleep_time = 4;
        while(retries)
        {
            if (-1 == bind(sd, (struct sockaddr *)&sin, sizeof(sin)) )
            {
                if ( EADDRINUSE == errno )
                {
                    Llog_debug("Socket not bound to port: %s ", 
                        strerror(errno));
                    sleep ( sleep_time );
                    sleep_time += sleep_time;
                    retries--;   
                }
                else
                {
                    Llog_error("Failed to bind socket to port: %s ", 
                        strerror(errno)); 
                    return(-1);
                }
            }
            else
            {
                retries = 0;
            }
            
            /* Possibly print a message stating if a re-try will be attampted   */
            if(retries)
            {
                Llog_debug("retrying...\n");
            }
            else
            {
               Llog_debug("giving up on connection\n");
            }
        }

        /* show that we are willing to listen */
        if (-1 == listen(sd, MAX_CLIENTS))
        {
            Llog_error("Listen on socket failed: %s\n",  
                strerror(errno)); 
            return(-1);
        }
        
        Llog_debug("Listening on socket...\n"); 
        return(sd);
}


/************************************************************************/
/*									*/
/*  sl_Wait4Client                                                      */
/*                                                                      */
/*  This will wait forever for a client                                 */
/*  then return an "SD" to the new client.  Using this routine is easy  */
/*  but what you can do with it is limited.                             */
/*                                                                      */
/*  Input Argument:                                                     */
/*      socket -- The socket that was returned by a call to             */
/*                sl_Setup_Server_Socket()                              */
/*                                                                      */
/************************************************************************/
/* Wait for then connect with one client.  */
int sl_Wait4Client(int socket)
{
    struct sockaddr_in	pin;
    int     		addrlen;
    int			new_sock;
    
    /* Check the argument */
    if (socket <= 0)
    {
        /* Invalid SD */
        Llog_warning("sl_Wait4Client called with invalid socket\n");
        return -1;
    }
       
    new_sock = accept(socket, (struct sockaddr *)&pin, &addrlen);

    /* accept returns -1 one error */
    if (new_sock == -1)
    {
        Llog_debug("accept returned -1, %s\n", strerror(errno));

        switch (errno)
        {
            case EWOULDBLOCK:
            {
		/* The  socket  is  marked  non-blocking  and	*/ 
		/* no connections are present to be accepted.	*/
		/* This is normal and not concidered and error.	*/
		return 0;
	    }
            case EBADF:         /* The descriptor is invalid.	*/
            case ENOTSOCK:      /* The descriptor references a 	*/
                		/* file, not a socket.		*/
            case EOPNOTSUPP:    /* The referenced socket is 	*/
                		/* not of type SOCK_STREAM.		*/
            case EFAULT:        /* The addr parameter is not in	*/
                                /* a writable part of the user	*/
                		/* address space.			*/
            default:
            {
                /* We should never get any of the above cases.	*/

                Llog_error("Accept on socket failed: %s\n", 
                    strerror(errno)); 
                LLOG_MARK;
            }

        /* Return Error	*/    
        return (-1);
	}
    }
    else
    {
        /* It worked!  We are connected to a new client */
        Llog_debug("conect to new client\n");
        
        /* Now we need to get a buffer that we will use for reading */
        /* data from this socket                                    */
       

        if( -1 == sd_new_sd(new_sock))
        {
            /* error */
            Llog_warning("sd_new_sd failed.\n");
            LLOG_MARK;
            return -1;
        } 
           
        return new_sock;
    }
}

